local lgr = love.graphics
local kbd = love.keyboard
local tmr = love.timer
local lfs = love.filesystem

local draw = lgr.drawq
if love.window then draw = lgr.draw end

local scrollingFood = {}
local turtle = {}
local turtles = {}

function createScrollingFoodBackground()
	for x = -8, 8 do
		scrollingFood[x] = {}
		for y = -8, 8 do
			scrollingFood[x][y] = {img = math.random(1,11), rot = (x * y) + y}
		end
	end

	for i = 1, 11 do
		turtles[i] = turtle:create(math.random(-8, 8), math.random(-8, 8), i)
	end
end

function drawScrollingFoodBackground()
	local dt = tmr.getDelta()

	for i, t in pairs(turtles) do
		t:update(dt)
	end
	lgr.push()
	lgr.translate(DRAW_WIDTH / 2 + (math.sin(tmr.getTime() / 10) * 100),DRAW_HEIGHT / 2 + (math.cos(tmr.getTime() / 20) * 100))
	lgr.scale(0.65)
	--lgr.rotate(math.rad(math.sin(tmr.getTime()/10) * 10))
	lgr.rotate(tmr.getTime()/10)
	for x = -8, 8 do
		for y = -8, 8 do
			lgr.setColor(255,255,255,50)
			scrollingFood[x][y].rot = scrollingFood[x][y].rot + dt
			local rot = math.rad(math.sin(scrollingFood[x][y].rot) * 20)
			rot = -tmr.getTime()/10
			if scrollingFood[x][y].img > 0 then
				draw(game.foodSheetLinear, game.foodQuad[scrollingFood[x][y].img], math.floor(x * 200), math.floor(y * 200), rot, 1, 1, 100, 100)
			end
		end
	end
	lgr.pop()
end

function turtle:create(x, y, id)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(x, y, id)
	return _i
end
function turtle:setup(x, y, id)
	v = v or {}
	self.x = x
	self.y = y
	self.dir = math.random(1,4)
	self.timer = 0
	self.id = id
end

function turtle:update(dt)
	local oldDir = self.dir
	local newDirList = {}
	for i = 1, 4 do
		if i ~= oldDir then
			newDirList[#newDirList+1] = i
		end
	end
	self.timer = self.timer - dt
	if self.timer <= 0 then
		local dir = newDirList[math.random(1,#newDirList)]
		if dir == 1 then
			if self.x > -8 then self.x = self.x - 1 end
		elseif dir == 2 then
			if self.y > -8 then self.y = self.y - 1 end
		elseif dir == 3 then
			if self.x < 8 then self.x = self.x + 1 end
		elseif dir == 4 then
			if self.y < 8 then self.y = self.y + 1 end
		end
		self.dir = dir
		self.timer = .5
		scrollingFood[self.x][self.y].img = self.id--math.random(1,11)
	end
end

function drawBackground(r,g,b)
	lgr.setColor(r,g,b)
	lgr.rectangle("fill", 0, 0, DRAW_WIDTH, DRAW_HEIGHT)
end
