local lgr = love.graphics
local kbd = love.keyboard

local draw = lgr.drawq
if love.window then draw = lgr.draw end

foodObject = {}
function foodObject:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v)
	return _i
end
function foodObject:setup(v)
	v = v or {}
	self.x = v.x or 0
	self.y = v.y or 0
	self.rad = 12
	self.speed = ((game.level.speedList[math.random(1, #game.level.speedList)] * 100))
	self.yspeed = 0
	-- if math.random(1,10) == 1 then self.yspeed = math.random(-100, 100) end
	self.id = v.id or "food"..math.random(10000,99999)

	self.spriteSheet = game.foodSheet
	self.quadList = game.foodQuad
	self.foodType = v.foodType or foodTypeList[math.random(1,#foodTypeList)]
	self.spinSpeed = math.rad(math.random(90,180) * chooseNegOrPos())
	self.rotation = 0

	self.track = v.track or 1

	self.frame = 0
end

function foodObject:update(dt)
	self.x = self.x + dt * self.speed
	self.y = self.y + dt * self.yspeed
	if self.x > game.level.bounds.width + (spriteSize / 2) then
		game.deadFoodList[#game.deadFoodList+1] = self.id
	end
	if self.y > game.level.bounds.height - (spriteSize / 2) or self.y < (spriteSize / 2) then
		self.yspeed = -self.yspeed
	end

	self.rotation = self.rotation + dt * self.spinSpeed
end

function foodObject:draw()
	local n = foodStats[self.foodType].quad
	local s = 0.18-- + (math.cos(self.rotation * 2) / 10)
	lgr.setColor(255,255,255)
	draw(self.spriteSheet, self.quadList[n], (self.x), (self.y), self.rotation, s, s, (spriteSize / 2), (spriteSize / 2))
	-- lgr.setColor(255,0,0,100)
	-- lgr.circle("fill", self.x, self.y, self.rad)
end

function foodObject:collect()
	-- player = player or 1
	local p = game.player

	p.calories = p.calories + foodStats[self.foodType].calories
	p.calorieCounter = p.calorieCounter + foodStats[self.foodType].calories

	-- Every 500 Calories, remove one Purple Pickle
	if p.calorieCounter >= 500 then
		p.rottenPickles = math.max(p.rottenPickles - 1, 0)

		p.calorieCounter = p.calorieCounter - 500
	end

	if foodStats[self.foodType].harmful then
		-- If food is a Purple Pickle, add to total and reset multiplier. Check for game over.
		p.foodSincePickle = 0
		p.rottenPickles = p.rottenPickles + 1
		game.multiplier = 1

		if p.rottenPickles > 5 and game.playmode == 1 then
			print("GAME OVERRRRRRR")
			playSound("gameover")
			changeState("gameover")
		else
			playSound("eatpickle")
		end
	else
		-- Else if normal food then add to score and calorie counter and update multiplier. If enough food is eaten, level up.
		-- 10 points x multiplier per food item. Maximum x100 multiplier.
		playSound("eatfood")
		local multiplier = 10 * multiplierList[math.ceil(game.multiplier)] * game.skill
		p.points = p.points + multiplier

		local fid = "floater_" .. game.level.totalfloaters+1
		game.floaters[fid] = floatPoint:create { x = self.x - self.rad, y = self.y - self.rad, value = multiplier, id = fid}

		p.foodSincePickle = p.foodSincePickle + 1
		game.level.totalFood = game.level.totalFood + 1
		p.foodCollected[#p.foodCollected+1] = self.foodType
		-- game.multiplierTick = game.multiplierTick + 1
		-- if game.multiplierTick > 5 then
			-- game.multiplierTick = 1
			game.multiplier = math.min(game.multiplier + 1, #multiplierList)
		-- end
		-- print(self.foodType)

		-- if game.level.finished and game.player.frame == 1 then
		-- 	print("Level up!")
		-- 	changeState("levelup")
		-- end
	end

	-- Add this food object to a list for removal at the end of the current frame.
	game.deadFoodList[#game.deadFoodList+1] = self.id
end

function spawnFood(track, foodType)
	if game.totalFoodSpawns < game.level.foodNeededForNext then
		game.totalFoodSpawns = game.totalFoodSpawns + 1
		local id = "food_"..game.totalFoodSpawns
		-- print(game.totalFoodSpawns, id)
		-- local y = math.random((25), (game.level.bounds.height) - (25))
		local s = game.level.bounds.height / 3
		local y = (track - 0.5) * s
		print(game.level.bounds.height,s,y)
		game.food[id] = foodObject:create {x = -(25), y = y, id = id, track = track}

		game.level.foodOnScreen = game.level.foodOnScreen + 1
	end
end

function destroyFood()
	for i = 1, #game.deadFoodList do
		game.totalFoodRemovals = game.totalFoodRemovals + 1

		game.level.track[game.food[game.deadFoodList[i]].track].active = false
		game.level.track[game.food[game.deadFoodList[i]].track].countdown = 0

		game.food[game.deadFoodList[i]] = nil
		print("Removing", game.deadFoodList[i])
		game.level.foodOnScreen = game.level.foodOnScreen - 1

	end
end

multiplierList = { 1, 2, 4, 8, 10, 20, 40, 80, 100 }

foodStats = {
	["French Fries"] = {
		quad = 1,
		calories = 10
	},
	["Hot Dog"] = {
		quad = 2,
		calories = 5
	},
	["Ice Cream Bar"] = {
		quad = 3,
		calories = 7
	},
	["Cheese Burger"] = {
		quad = 4,
		calories = 20
	},
	["Hamburger"] = {
		quad = 10,
		calories = 6
	},
	["Mug of Beer"] = {
		quad = 5,
		calories = 5
	},
	["Pizza Slice"] = {
		quad = 6,
		calories = 10
	},
	["Root Beer"] = {
		quad = 7,
		calories = 3
	},
	["Cola"] = {
		quad = 8,
		calories = 4
	},
	["Green Pickle"] = {
		quad = 11,
		calories = 1,
	},
	["Purple Pickle"] = {
		quad = 9,
		calories = 0,
		harmful = true
	},
}

helpFoodList = {}
local n = 1
for i, f in pairs(foodStats) do
	helpFoodList[n] = {
		name = i, quad = f.quad, calories = f.calories
	}
	n = n + 1
end

table.sort( helpFoodList,
	function(a, b)
		-- if a.calories ~= b.calories then return a.calories < b.calories end
		return a.calories < b.calories
	end
)

foodTypeList = {}
for i, f in pairs(foodStats) do
	foodTypeList[#foodTypeList+1] = i
end
