local lgr = love.graphics
local kbd = love.keyboard
local tmr = love.timer
local lfs = love.filesystem

function isNine()
	if love._version_major >= 0 and love._version_minor >= 9 then return true end
	return false
end

local draw = lgr.drawq
if isNine() then draw = lgr.draw end

function chooseNegOrPos()
	local neg = { -1, 1 }
	return neg[math.random(1,2)]
end

function overlap(x1,y1,w1,h1, x2,y2,w2,h2)
	return not (x1+w1 < x2  or x2+w2 < x1 or y1+h1 < y2 or y2+h2 < y1)
end

function math.distanceFrom(x1,y1,x2,y2) return math.sqrt((x2 - x1) ^ 2 + (y2 - y1) ^ 2) or 0 end

function math.findAngle(x1, y1, x2, y2, rad)
	local t = math.deg(math.atan2(y1 - y2, x2 - x1))
	--if t < 0 then t = t + 360 end
	if rad then
		return math.rad(t)
	else
		return t
	end
end

function changeState(state)
	local oldState = game.state
	game.state = state

	if oldState == "menu" and state == "game" then
		--Starting New Game
		game.menu:setCurrentPage("main", true)

		playSound("spawnfood")
		game.paused = false

		game.multiplier = 1
		game.multiplierTick = 0

		game.level = level:load(1)

		if game.playmode == 1 then
			game.countdownTimer = 0
		elseif game.playmode == 2 then
			game.countdownTimer = 180
		end

		game.totalFoodSpawns = 0
		game.totalFoodRemovals = 0

		game.player = playerObject:create {x = game.level.bounds.width * 0.8, y = game.level.bounds.height / 2}
		game.food = {}
		game.floaters = {}
	elseif state == "howtoplay" and oldState == "menu" then
		game.howToTimer = 0
	elseif state == "menu" then
		game.intermissionTimer = 0
		game.paused = false

		game.level = nil

		game.skill = 0

		game.player = nil
		game.food = {}
		game.floaters = {}
	elseif oldState == "game" and state == "levelup" then
		playSound("levelup")
		game.intermissionTimer = 2

		game.totalFoodSpawns = 0
		game.totalFoodRemovals = 0

		game.food = {}
		game.floaters = {}
	elseif oldState == "levelup" and state == "game" then
		game.level.totalFood = 0
		game.level = level:load(game.skill + 1)
		game.food = {}
	elseif oldState == "game" and state == "gameover" then
		game.intermissionTimer = 3
		if highScoreTable[game.playmode].score < tonumber(game.player.points) then
			highScoreTable[game.playmode].score = tonumber(game.player.points)
			highScoreTable[game.playmode].calories = tonumber(game.player.calories)
			highScoreTable[game.playmode].level = tonumber(game.skill)
			-- highScoreTable[game.playmode].time = tonumber(game.skill)
		end
	end
end

function printShadowed(v)
	v = v or {}
	local x = v.x or 0
	local y = v.y or 0
	local sOff = v.shadowOffset or -3
	local fontname = v.font or "dialog"
	local align = v.align or "left"
	local width = v.width or lgr.getWidth
	local text = v.text or ""
	local color = v.color or {255,255,255}
	local shadowColor = v.shadowColor or {0,0,0,100}
	lgr.setFont(font[fontname])
	lgr.setColor(shadowColor)
	lgr.printf(text, x, y+sOff, width, align)
	lgr.setColor(color)
	lgr.printf(text, x, y, width, align)
end

function string.formatNumber(number)
	number = number or 0
	return (string.format("%d", number):reverse():gsub( "(%d%d%d)" , "%1," ):reverse():gsub("^(-?),","%1"))
end

function string.formatTime(t, p, ms)
	local h = math.floor(math.mod(t/60/60,60))
	local zero = ""
	if h > 0 then zero = "0" end
	local m = string.sub(zero .. math.floor(math.mod(t/60,60)),-2)
	local s = string.sub("0" .. math.floor(math.mod(t,60)),-2)
	local r = ""
	if p then
		if h > 0 then r = h .. ":" .. m else r = m .. ":" .. s end
	else
		r = h .. ":" .. m .. ":" .. s
	end
	if ms then r = r .. (t - math.floor(t)) end
	return r
end

function changeScreenMode(m)
	local mode = 1
	if type(m) == "number" then
		mode = m
	elseif type(m) == "boolean" then
		if m == true then
			mode = currentScreenMode + 1
		else
			mode = currentScreenMode - 1
		end
	end
	currentScreenMode = mode
	if currentScreenMode > #screenModes then currentScreenMode = 1 end
	if currentScreenMode < 1 then currentScreenMode = #screenModes end
	if isNine() then
		local w, h = 1280, 720--screenModes[currentScreenMode].width, screenModes[currentScreenMode].height
		love.window.setMode(w, h, {fullscreen = false, vsync = preferences.vsync, resizable = true, fsaa = preferences.fsaa})
	else
		lgr.setMode(screenModes[currentScreenMode].width, screenModes[currentScreenMode].height, false, preferences.vsync, preferences.fsaa)
		addMessage("Changed mode to " .. screenModes[currentScreenMode].width .. "x" .. screenModes[currentScreenMode].height)
	end
	updateWindowSize()

end

function setFullscreen(f)
	game.screen.fullscreen = f
	if f then
		if isNine() then
			love.window.setMode(screenModes[1].width, screenModes[1].height, {fullscreen = true, vsync = preferences.vsync, resizable = false, fsaa = preferences.fsaa})
		else
			lgr.setMode(screenModes[1].width, screenModes[1].height, true, preferences.vsync, preferences.fsaa)
		end
		updateWindowSize()
		addMessage("Set window to Fullscreen")
	else
		changeScreenMode(currentScreenMode)
	end
end

function playSound(s)
	if preferences.soundOn then
		soundmanager:play(sfx[s])
	end
end

function onOff(oo)
	local t = {[true] = "on", [false] = "off"}
	return t[oo]
end

function updateWindowSize(setWinMode)
	local w, h = lgr.getWidth() / DRAW_WIDTH, lgr.getHeight() / DRAW_HEIGHT
	print("Window scale:", w, h)
	game.screen.windowScale = {}
	game.screen.windowScale.w = w
	game.screen.windowScale.h = w

	if setWinMode then
		love.window.setMode(w * DRAW_WIDTH, w * DRAW_HEIGHT, {fullscreen = false, vsync = true, resizable = true, fsaa = 4})
	end

	game.screen.windowScale.padX = (lgr.getWidth() - (game.screen.windowScale.w * DRAW_WIDTH)) / 2
	game.screen.windowScale.padY = (lgr.getHeight() - (game.screen.windowScale.h * DRAW_HEIGHT)) / 2
	print("Padding:", game.screen.windowScale.padX, game.screen.windowScale.padY)
end

function loadPreferences()
	print("Loading Preferences")
	local s = lfs.load("settings/preferences.lua")()
	preferences = s

	if lfs.exists("preferences.lua") then
		local t = lfs.load("preferences.lua")()
		for i, p in pairs(t) do
			preferences[i] = p
		end
	end

	highScoreTable = {
		[1] = {
			score = 0,
			calories = 0,
			level = 0,
			-- time = 0
		},
		[2] = {
			score = 0,
			calories = 0,
			level = 0,
			-- time = 0
		}
	}

	if lfs.exists("highscores.lua") then
		highScoreTable = lfs.load("highscores.lua")()
	end

end

function savePreferences()
	-- print("\nSaving Preferences...")
	local t = "return {\n"
	for i, p in pairs(preferences) do
		local pp = tostring(p)
		if type(p) == "string" then
			pp = "'" .. pp .. "'"
		end
		t = t .. "	" .. i .. " = " .. pp .. ",\n"
	end
	t = t .. "}"
	lfs.write("preferences.lua", t, all)

	local t = "return {\n"
	for i = 1, 2 do
		t = t .. "\t["..i.."] = {\n"
		t = t .. "\t\tscore = " .. highScoreTable[i].score .. ",\n"
		t = t .. "\t\tcalories = " .. highScoreTable[i].calories .. ",\n"
		t = t .. "\t\tlevel = " .. highScoreTable[i].level .. ",\n"
		t = t .. "\t},\n"
	end
	t = t .. "}"

	lfs.write("highscores.lua", t, all)

	-- print(t .. "\nDone!\n")
end
