local lgr = love.graphics
local tmr = love.timer

local function isChar(k, chars)
    local ic = false
    chars = chars or DEFAULT_CHAR_STRING
    for i = 1, #chars do
        if string.sub(chars, i, i) == k then
            ic = true
        end
    end
    return ic
end

--  dP""b8 88   88 88     .dP"Y8 Yb  dP .dP"Y8 888888 888888 8b    d8
-- dP   `" 88   88 88     `Ybo."  YbdP  `Ybo."   88   88__   88b  d88
-- Yb  "88 Y8   8P 88     o.`Y8b   8P   o.`Y8b   88   88""   88YbdP88
--  YboodP `YbodP' 88     8bodP'  dP    8bodP'   88   888888 88 YY 88
guiSystem = {}
local guiElement = {}
function guiSystem:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v)
	return _i
end
function guiSystem:setup(v)
	v = v or {}
	self.page = {}
	self.currentPage = ""
	self.width = (v.width or 400)
	self.height = 0
	self.align = v.align or "left"
	self.escapeAction = v.escapeAction or function() end
	self.trail = {}
	self.trailLoc = 0
	self.transition = 0
	self.transitionTo = 0
	self.transitionSpeed = preferences.menuSpeed or 3
	self.maxperpage = 6
	self.transitionCallback = nil

	v.snap = v.snap or {}
	self.snap = {
		x = v.snap[1] or "center",
		y = v.snap[2] or "center"
	}

	v.margin = v.margin or {}
	self.margin = {
		l = v.margin[1] or 0,
		t = v.margin[2] or 0,
		r = v.margin[3] or 0,
		b = v.margin[4] or 0
	}
	self.fillHeight = v.margin[5] or "fit"
	self.fillWidth = v.margin[6] or "fit"

	self.bgcolor = v.bgcolor or cWetAsphalt
end

function guiSystem:getHeight()
	return self.height
end

function guiSystem:update(dt)
	-- local v = 0.5 - 0.5 * math.cos(tmr.getTime() * 5)
	if self.transition > self.transitionTo then self.transition = math.max(self.transition - dt * self.transitionSpeed, 0) end
	if self.transition < self.transitionTo then self.transition = math.min(self.transition + dt * self.transitionSpeed, 1) end

	if self.page[self.currentPage] then
		self.width = self.page[self.currentPage].width
		if self.transition == 1 then
			for i, l in pairs(self.page[self.currentPage].list) do
				local li = self.page[self.currentPage].list[i]
				if li.selAnimTime > li.selAnimTo then
					li.selAnimTime = math.max(li.selAnimTime - dt * (li.selAnimSpeed * 2), 0) end
				if li.selAnimTime < li.selAnimTo then
					li.selAnimTime = math.min(li.selAnimTime + dt * li.selAnimSpeed, 1) end
				l:update(dt)
			end
		end
	end

	local sX, sY = self.snap.x, self.snap.y
	local mL, mT, mR, mB = self.margin.l, self.margin.t, self.margin.r, self.margin.b
	if sX == "left" then
		self.x = mL
	elseif sX == "center" then
		self.x = (DRAW_WIDTH - self.width) / 2
	elseif sX == "right" then
		self.x = DRAW_WIDTH - (mR + self.width)
	end

	if sY == "top" then
		self.y = mT
	elseif sY == "center" then
		self.y = (DRAW_HEIGHT - self.height) / 2
	elseif sY == "bottom" then
		self.y = DRAW_HEIGHT - (mB + self.height)
	end
end

function guiSystem:draw(mx,my)
	if _dbg.on then
		lgr.setColor(color.White)
		setFont("menu")
		lgr.print(self.trailLoc .. " - " .. tostring(self.currentPage), 0, 0)
		if self.trailLoc > 0 then
			for i = 1, #self.trail do
				if i < self.trailLoc then
					lgr.setColor(255,255,255)
				elseif i == self.trailLoc then
					lgr.setColor(255,0,0)
				else
					lgr.setColor(255,255,255,100)
				end
				lgr.print(i .. ") \"" .. tostring(self.trail[i]) .. "\"", 10, 14 * i)
			end
		end

		local tl = ""
		if self.trail[self.trailLoc] then tl = self.trail[self.trailLoc] else tl = "NIL" end
		local cp = ""
		if self.currentPage then cp = self.currentPage else cp = "NIL" end
		lgr.print("Current: " .. cp .. "\nPrevious: " .. tl, 150, 0)
	end

	local lpage = self.currentPage
	self:drawPage(lpage)
end

function guiSystem:drawPage(lpage)
	if lpage ~= nil then
		lgr.setLineStyle("rough")
		local trans = self.transition
		if self.page[lpage] then
			lgr.push()
			local w = self.margin.r + self.width + self.margin.l + 30
			local o = -(w - (w * easing.outQuad(trans, 0, 1, 1)))
			local sw, sh = 0,0--self.width, self.height
			lgr.translate( o + (self.x + (sw)), self.y + (sh) )
			-- lgr.scale(easing.outBack(trans, 0, 1, 1, 2),1)
			-- lgr.rotate(-(1-easing.outSine(trans, 0, 1, 1))/10)
			lgr.translate( -(sw), -(sh) )

			-- local s = self.page[lpage].margin

			local by = -self.margin.t
			local bh = self:getHeight()+self.margin.t+self.margin.b
			if self.fillHeight then
				by = -(self.y + 100)
				bh = DRAW_HEIGHT + 200
			end
			lgr.setColor(self.bgcolor)
			lgr.rectangle("fill", -self.margin.l, by, self.width+self.margin.l+self.margin.r, bh)

			local header = self.page[lpage].header or ""
			if header ~= "" then
				local a = "center"--self.align
				setFont("menu_header") --self.page[lpage].font
				lgr.setColor(0,0,0,100)
				lgr.printf(header, 0, 2, self.width, a)
				lgr.setColor(color.SunFlower)
				lgr.printf(header, 0, 0, self.width, a)
			end

			local j = 1
			local x = 0
			local y = self:getHeadHeight()
			for i, l in pairs(self.page[lpage].list) do
				local focused = false
				setFont(self.page[lpage].font)
				if self.page[lpage].selected == j then
					setFont(self.page[lpage].fontsel)
				end

				local f = self.page[lpage].list[i].selAnimTime
				local a = easing.outQuad(f, 0, 1, 1, 2)

				if a > 0 then
					if self.page[lpage].selected == j then focused = true end

					local w = self.page[lpage].list[i].width
					local h = self.page[lpage].spacing
					local p = self.page[lpage].list[i].padding

					if self.snap.x == "left" then
						pts = {
							x - self.margin.l,
							y,
							(self.margin.r + self.width),
							y,
							(self.margin.r + self.width),
							y + h,
							x - self.margin.l,
							y + h
						}
					else
						pts = {
							x - self.margin.l,
							y,
							(self.margin.r + self.width),
							y,
							(self.margin.r + self.width),
							y + h,
							x - self.margin.l,
							y + h
						}
					end

					-- lgr.setColorMode("combine")
					-- lgr.setBlendMode("additive")
					lgr.setColor(255,255,255,150*a)
					lgr.polygon("fill", pts)
					-- lgr.setBlendMode("alpha")
					-- lgr.setColorMode("modulate")
				end

				if focused then
					lgr.setColor(color.SunFlower)
					l:draw(self, x, y+2, true, focused)
				else
					lgr.setColor(0,0,0,100)
					l:draw(self, x , y+2, true, focused)
				end

				self.selectedColor = color.Pumpkin
				lgr.setColor(self.page[lpage].list[i].textColor or color.White)
				if self.page[lpage].selected == j then
					setFont(self.page[lpage].fontsel)
					lgr.setColor(self.selectedColor)
				end
				l:draw(self, x, y, false, focused)
				y = y + self:getListItemHeight(lpage, j) + self.page[lpage].padding
				j = j + 1
			end
			lgr.pop()
		end
	end
end

function guiSystem:getListItemHeight(page, i)
	local p = self.page[page].padding
	local h = self.page[page].list[i].height + p
	return h
end

function guiSystem:drawDescription(x,y)
	if self.page[self.currentPage] then
		local sel = self.page[self.currentPage].selected
		if sel > 0 then
			lgr.push()
			lgr.translate(x,y)
			local t = self.page[self.currentPage].list[sel].desc or ""
			lgr.setColor(255,255,255)
			setFont("menu_description")
			lgr.print(t, 10, 10)
			lgr.pop()
		end
	end
end

function guiSystem:setCurrentPage(page, clear)
	self.transitionTo = 0
	newTimer(1/self.transitionSpeed, function()
		self:changePage(page, clear)
		self.transitionTo = 1
		print(page)
	end)
end

function guiSystem:back()
	if self.trailLoc > 1 then
		self.transitionTo = 0
		newTimer(1/self.transitionSpeed, function()
			self:changePage("_back")
			self.transitionTo = 1
		end)
	end
end

function guiSystem:changePage(page, clear)
	if page == nil or page == "" then
		self.currentPage = page
	elseif page == "_back" then
		self.currentPage = self.trail[self.trailLoc]
		self.trailLoc = self.trailLoc - 1

		self:recalculateHeight()

		if self.currentPage and self.currentPage ~= "" then
			for i, m in pairs(self.page[self.currentPage].list) do
				m.selAnimTime = 0
				m.selAnimTo = 0
				if m.init then m.value = m.init() end
			end
			if self.page[self.currentPage].changeTo then self.page[self.currentPage].changeTo() end

			self:selectionAnimIn(true)
		end
	else
		if clear then
			self.trail = {}
			self.trailLoc = 1
			self.trail[1] = self.currentPage
		else
			self.trailLoc = self.trailLoc + 1
			self.trail[self.trailLoc] = self.currentPage
		end
		self.currentPage = page

		self:recalculateHeight()

		if self.currentPage and self.currentPage ~= "" then
			for i, m in pairs(self.page[page].list) do
				m.selAnimTime = 0
				m.selAnimTo = 0
				if m.init then m.value = m.init() end
			end
			if self.page[self.currentPage].changeTo then self.page[self.currentPage].changeTo() end

			self:selectionAnimIn(true)
		end
	end
end

function guiSystem:getHeadHeight()
	if self.currentPage and self.currentPage ~= "" then
		local headHeight = 0
		if self.page[self.currentPage].header ~= "" then
			headHeight = (getFontHeight("menu") + self.page[self.currentPage].padding)
		end
		return headHeight
	end
end

function guiSystem:recalculateHeight()
	if self.currentPage and self.currentPage ~= "" then
		self.height = self:getHeadHeight()
		for i = 1, #self.page[self.currentPage].list do
			self.height = self.height + self:getListItemHeight(self.currentPage, i) + self.page[self.currentPage].padding
		end
		self.height = self.height - self.page[self.currentPage].padding
	end
end

function guiSystem:selectionAnimIn(set)
	local page = self.currentPage
	local sel = self.page[page].selected
	if sel > 0 then
		self.page[page].list[sel].selAnimTo = 1
		if set then
			self.page[page].list[sel].selAnimTime = 1
		end
	end
end
function guiSystem:selectionAnimOut(set)
	local page = self.currentPage
	local sel = self.page[page].selected
	if sel > 0 then
		self.page[page].list[sel].selAnimTo = 0
		if set then
			self.page[page].list[sel].selAnimTime = 0
		end
	end
end

function guiSystem:addPage(page, v)
	local _font = v.font or "menu"
	local _fontsel = v.fontsel or "menu"
	local _header = v.header or ""
	local _spacing = math.floor((v.spacing or 4))
	local _align = v.align or self.align
	local _padding = math.floor((v.padding or 0))

	-- if self.page[page] == nil then
	-- 	print("Creating page")
	-- else
	-- 	print("Page exists")
	-- end

	self.page[page] = {}
	self.page[page].header = _header
	self.page[page].selected = 1
	self.page[page].list = {}
	self.page[page].font = _font
	self.page[page].fontsel = _fontsel
	self.page[page].spacing = _spacing
	self.page[page].align = _align
	self.page[page].padding = _padding
	self.page[page].width = (v.width or self.width)
	self.page[page].changeTo = v.changeTo or function() end
end

function guiSystem:addListItem(page, v, enabled)
	local _title = v.title or "[NO TITLE OOPS]"
	local _func = v.func or function() end
	local _init = v.init or function() end
	local _type = v.type or "label"
	local _desc = v.desc or ""
	local _enabled = enabled or true
	local _tcolor = v.textColor or {255,255,255}
	-- print(_title, "Enabled", _enabled or enabled)
	_type = string.lower(_type)

	local p = self.page[page].list
	local n = #self.page[page].list + 1

	local c = v.value
	if v.init then c = v.init() end

	p[n] = guiElement[_type]:create({
		title = _title,
		func = _func,
		init = _init,
		type = _type,
		align = self.page[page].align,
		desc = _desc,
		enabled = _enabled,
		selAnimSpeed = 1.5,
		parent = self,
		textColor = _tcolor
	})
	if _type == "checkbox" then
		p[n].value = c or false
		p[n].appearance = v.appearance or "ONOFF"
	elseif _type == "slider" then
		p[n].value = c or 0
		p[n].minval = v.minval or 0
		p[n].maxval = v.maxval or 100
		p[n].step = v.step or 10
		p[n].percentage = v.percentage or false
	elseif _type == "option" then
		p[n].process = v.process or function() end
		p[n].value = v.value or 1
		p[n].list = v.list or {"Nothing"}
		local w = 0
		for i = 1, #p[n].list do
			local iw = getFontWidth("menu", p[n].list[i])
			if iw > w then w = iw end
		end
		p[n].boxWidth = (math.ceil((w + 30) / 10) * 10) or 100
	elseif _type == "textinput" then
		p[n].value = c or ""
		p[n].chars = v.chars
		p[n].maxlength = v.maxlength or 20
		p[n].default = v.default or "ss"
	elseif _type == "keycapture" then
		p[n].value = c or ""
	end
end

function guiSystem:addSeparator(page)
	local p = self.page[page].list
	local n = #self.page[page].list + 1
	p[n] = guiElement["separator"]:create({ type = "separator" })
end

function guiSystem:clearSelection()
	self.trail = {}
	self.trailLoc = 1
	self.trail[1] = self.currentPage
	self:selectionAnimOut()
end

function guiSystem:keypressed(k, u)
	if self.page[self.currentPage] then
		local sel = self.page[self.currentPage].selected
		local allow = true
		if sel > 0 and self.transition == 1 and self.page[self.currentPage].list[sel].enabled == true then
			allow = self.page[self.currentPage].list[sel]:keypressed(k, u)
		end

		if allow then
			self:selectionAnimOut()
			if k == "up" then
				self:up()
			elseif k == "down" then
				self:down()
			end
			self:selectionAnimIn()
		end
	end
end

function guiSystem:up()
	local n = self.page[self.currentPage].selected
	local i = 1
	local s = ""
	if n > 1 then
		s = self.page[self.currentPage].list[n-1].type
		if s == "separator" then i = 2 end
	end
	self.page[self.currentPage].selected = self.page[self.currentPage].selected - i
	if self.page[self.currentPage].selected < 1 then
		self.page[self.currentPage].selected = #self.page[self.currentPage].list
	end
end
function guiSystem:down()
	local n = self.page[self.currentPage].selected
	local i = 1
	local s = ""
	if n < #self.page[self.currentPage].list then
		s = self.page[self.currentPage].list[n+1].type
		if s == "separator" then i = 2 end
	end
	self.page[self.currentPage].selected = self.page[self.currentPage].selected + i
	if self.page[self.currentPage].selected > #self.page[self.currentPage].list then
		self.page[self.currentPage].selected = 1
	end
end



-- 88        db    88""Yb 888888 88         88""Yb 88   88 888888 888888  dP"Yb  88b 88
-- 88       dPYb   88__dP 88__   88         88__dP 88   88   88     88   dP   Yb 88Yb88
-- 88  .o  dP__Yb  88""Yb 88""   88  .o     88""Yb Y8   8P   88     88   Yb   dP 88 Y88
-- 88ood8 dP""""Yb 88oodP 888888 88ood8     88oodP `YbodP'   88     88    YbodP  88  Y8
guiElement.label = {}
function guiElement.label:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v)
	return _i
end
function guiElement.label:setup(v)
	self.title = v.title
	self.func = v.func
	self.init = v.init
	self.type = v.type
	self.align = v.align
	self.desc = v.desc
	self.padding = v.padding
	self.selAnimTime = 0
	self.selAnimSpeed = v.selAnimSpeed
	self.selAnimTo = 0
	self.width = getFontWidth("menu", self.title)
	self.height = getFontHeight("menu")
	self.parent = v.parent
	self.enabled = v.enabled
	self.textColor = v.textColor
end
function guiElement.label:update(dt)

end
function guiElement.label:draw(parent, x, y)
	if self.enabled == false then
		lgr.setColor(100,100,100)
	end
	lgr.printf(self.title, x, y, parent.width, self.align)
end
function guiElement.label:keypressed(k, u)
	if k == "return" or k == " " then
		self.func()
	elseif k == "escape" then
		self.parent:back()
	end
	return true
end



--  dP""b8 88  88 888888  dP""b8 88  dP 88""Yb  dP"Yb  Yb  dP
-- dP   `" 88  88 88__   dP   `" 88odP  88__dP dP   Yb  YbdP
-- Yb      888888 88""   Yb      88"Yb  88""Yb Yb   dP  dPYb
--  YboodP 88  88 888888  YboodP 88  Yb 88oodP  YbodP  dP  Yb
guiElement.checkbox = {}
function guiElement.checkbox:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v)
	return _i
end
function guiElement.checkbox:setup(v)
	self.title = v.title
	self.func = v.func
	self.init = v.init
	self.type = v.type
	self.appearance = v.appearance or "ONOFF"
	self.value = v.value or false
	self.desc = v.desc
	self.padding = v.padding
	self.selAnimTime = 0
	self.selAnimSpeed = v.selAnimSpeed
	self.selAnimTo = 0
	self.width = v.parent.width--font["menu"]:getWidth(self.title)
	self.height = getFontHeight("menu")
	self.parent = v.parent
	self.enabled = v.enabled
	self.textColor = v.textColor
	-- print(self.enabled)
end
function guiElement.checkbox:update(dt)

end
function guiElement.checkbox:draw(parent, x, y, shadow, focused)
	shadow = shadow or false
	local a = "left"
	local ans
	if self.enabled == false then
		lgr.setColor(100,100,100)
	end
	if self.appearance == "ONOFF" then
		ans = { [true] = "ON", [false] = "OFF" }
	else
		ans = { [true] = "YES", [false] = "NO" }
	end
	lgr.printf(self.title, x, y, parent.width, a)

	local r,g,b,a = lgr.getColor()

	if not shadow then
		local c = { [true] = color.Nephritis, [false] = color.Pomegranate }
		lgr.setColor(c[self.value])
	else
		if focused then
			local c = { [true] = color.Emerland, [false] = color.Alizarin }
			lgr.setColor(c[self.value])
		else
			-- local c = { [true] = cEmerland, [false] = cAlizarin }
			lgr.setColor(0,0,0,100)
		end
	end

	if self.enabled == false then
		lgr.setColor(100,100,100)
	end

	lgr.push()
	lgr.translate(parent.width - 110 + x, 0 + y)
	setFont("menu")
	lgr.printf(ans[self.value], 0, 0, 110, "center")
	lgr.setColor(r,g,b,a)
	lgr.printf("<", 0, 0, 110, "left")
	lgr.printf(">", 0, 0, 110, "right")
	lgr.pop()
end
function guiElement.checkbox:keypressed(k, u)
	if k == "return" or k == " " or k == "left" or k == "right" then
		self.value = not self.value
		self.func(self.value)
	elseif k == "escape" then
		self.parent:back()
	end
	return true
end



-- .dP"Y8 88     88 8888b.  888888 88""Yb
-- `Ybo." 88     88  8I  Yb 88__   88__dP
-- o.`Y8b 88  .o 88  8I  dY 88""   88"Yb
-- 8bodP' 88ood8 88 8888Y"  888888 88  Yb
guiElement.slider = {}
function guiElement.slider:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v)
	return _i
end
function guiElement.slider:setup(v)
	self.title = v.title
	self.func = v.func
	self.init = v.init
	self.type = v.type
	self.value = v.value or false
	self.percentage = v.percentage or false
	self.desc = v.desc
	self.padding = v.padding
	self.selAnimTime = 0
	self.selAnimSpeed = v.selAnimSpeed
	self.selAnimTo = 0
	self.width = v.parent.width--font["menu"]:getWidth(self.title)
	self.height = getFontHeight("menu")
	self.parent = v.parent
	self.enabled = v.enabled
	self.textColor = v.textColor
end
function guiElement.slider:update(dt)

end
function guiElement.slider:draw(parent, x, y, shadow, focused)
	lgr.setLineWidth(1)
	shadow = shadow or false
	local a = "left"
	lgr.printf(self.title, x, y, parent.width, a)
	lgr.push()
	lgr.translate(parent.width - 110 + x, 0 + y)

	local n = math.round(self.value,1)
	local t = n
	if self.percentage then
		t = (n * 100) .. "%"
	end

	setFont("menu")
	lgr.printf(t, 0, 0, 110, "center")
	if self.value > self.minval then lgr.printf("<", 0, 0, 110, "left") end
	if self.value < self.maxval then lgr.printf(">", 0, 0, 110, "right") end
	lgr.pop()
end
function guiElement.slider:keypressed(k, u)
	if k == "left" then
		self.value = self.value - self.step
		if self.value < self.minval then self.value = self.minval end
	elseif k == "right" then
		self.value = self.value + self.step
		if self.value > self.maxval then self.value = self.maxval end
	elseif k == "escape" then
		self.parent:back()
	end
	self.func(self.value)
	return true
end



--  dP"Yb  88""Yb 888888 88  dP"Yb  88b 88     88     88 .dP"Y8 888888
-- dP   Yb 88__dP   88   88 dP   Yb 88Yb88     88     88 `Ybo."   88
-- Yb   dP 88"""    88   88 Yb   dP 88 Y88     88  .o 88 o.`Y8b   88
--  YbodP  88       88   88  YbodP  88  Y8     88ood8 88 8bodP'   88
guiElement.option = {}
function guiElement.option:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v)
	return _i
end
function guiElement.option:setup(v)
	self.title = v.title
	self.func = v.func
	self.init = v.init
	self.process = v.process
	self.type = v.type
	self.value = v.value or 1
	self.list = v.list or {"Nothing"}
	self.desc = v.desc
	self.padding = v.padding
	self.selAnimTime = 0
	self.selAnimSpeed = v.selAnimSpeed
	self.selAnimTo = 0
	self.width = v.parent.width--font["menu"]:getWidth(self.title)
	self.height = getFontHeight("menu")
	self.parent = v.parent
	self.enabled = v.enabled
	self.textColor = v.textColor
	-- print(self.list[self.value])
end
function guiElement.option:update(dt)

end
function guiElement.option:draw(parent, x, y, shadow, focused)
	shadow = shadow or false
	local a = "left"
	lgr.printf(self.title, x, y, parent.width, a)
	lgr.push()
	lgr.translate(parent.width - self.boxWidth + x, 0 + y)
	setFont("menu")
	lgr.printf(tostring(self.list[self.value]), 0, 0, self.boxWidth, "center")
	if self.value > 1 then lgr.printf("<", 0, 0, self.boxWidth, "left") end
	if self.value < #self.list then lgr.printf(">", 0, 0, self.boxWidth, "right") end
	lgr.pop()
end
function guiElement.option:keypressed(k, u)
	if k == "left" then
		self.value = self.value - 1
		if self.value < 1 then self.value = 1 end
	elseif k == "right" then
		self.value = self.value + 1
		if self.value > #self.list then self.value = #self.list end
	elseif k == "escape" then
		self.parent:back()
	end
	self.func(self.value)
	self.process(self.list[self.value])
	return true
end



-- 888888 888888 Yb  dP 888888     88 88b 88 88""Yb 88   88 888888
--   88   88__    YbdP    88       88 88Yb88 88__dP 88   88   88
--   88   88""    dPYb    88       88 88 Y88 88"""  Y8   8P   88
--   88   888888 dP  Yb   88       88 88  Y8 88     `YbodP'   88
guiElement.textinput = {}
function guiElement.textinput:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v)
	return _i
end
function guiElement.textinput:setup(v)
	self.title = v.title
	self.func = v.func
	self.init = v.init
	self.type = v.type
	self.value = v.value or ""
	self.maxlength = v.maxlength or 20
	self.desc = v.desc
	self.padding = v.padding
	self.selAnimTime = 0
	self.selAnimSpeed = v.selAnimSpeed
	self.selAnimTo = 0
	self.width = v.parent.width--font["menu"]:getWidth(self.title)
	self.height = getFontHeight("menu")
	self.boxWidth = 220
	self.boxHeight = getFontHeight("menu")
	self.chars = v.chars
	self.caretLeft = 0
	self.caret = ""
	self.default = v.default
	self.boxValue = ""
	self.parent = v.parent
	self.enabled = v.enabled
	self.textColor = v.textColor
end
function guiElement.textinput:update(dt)
	local val = self.value or self.default
	local isDefault = false
	if self.value == "" then
		val = self.default
		isDefault = true
	end
	self.boxValue = val
	self.isDefault = isDefault
	self.caret = "|"
	self.caretLeft = getFontWidth("menu", self.value) or 0
	if math.cos(love.timer.getTime() * 10) >= 0 then self.caret = "" end
end
function guiElement.textinput:draw(parent, x, y, shadow, focused)
	lgr.setLineWidth(1)
	shadow = shadow or false
	lgr.printf(self.title, x, y, parent.width, "left")
	local r,g,b,a = lgr.getColor()
	local cL = self.caretLeft or 0
	local tx = -math.max(0,cL-self.boxWidth)

	lgr.push()
	lgr.translate(0,1.5)
	if not shadow then
		lgr.setColor(0,0,0,50)
		lgr.rectangle("fill", x+(parent.width-self.boxWidth)-4, y-2+1, self.boxWidth+8, self.boxHeight+2)
		local a = 0.25
		if focused then a = 1 end
		lgr.setColor(255,255,255,255*a)
		lgr.rectangle("line", x+(parent.width-self.boxWidth)-4, y-2+1, self.boxWidth+8, self.boxHeight+2)
	end

	if parent.transition == 1 then
		lgr.setScissor(parent.x + x+(parent.width-self.boxWidth), parent.y + y, self.boxWidth, self.boxHeight)

		if not shadow then
			lgr.setColor(color.Asbestos)
			lgr.print(self.boxValue, x+(parent.width-self.boxWidth+tx), y+1)
			lgr.setColor(color.Clouds)
			if self.isDefault then lgr.setColor(160,160,160) end
			setFont("menu")
			lgr.print(self.boxValue, x+(parent.width-self.boxWidth+tx), y)
		end

		lgr.setScissor()
	end

	if focused and not shadow then
		lgr.setColor(r,g,b,a)
		lgr.print(self.caret, x+(parent.width-self.boxWidth+tx) + self.caretLeft, y)
	end

	lgr.pop()
end
function guiElement.textinput:keypressed(k, u)
	local sval = self.value
	if isChar(k, self.chars) then
		local c = ""
		if pcall(string.char, u) then
			c = tostring(string.char(u))
			-- print(c, k, u)
		end
		if #("x"..sval.."x") - 2 < self.maxlength then
			-- if shiftHeld() then char = string.upper(char) end
			self.value = sval .. c
		end
	elseif k == "backspace" then
		self.value = string.sub(sval, 0, #sval - 1) or ""
	elseif k == "left" then

	elseif k == "right" then

	elseif k == "escape" then
		self.parent:back()
	end
	self.func(self.boxValue)
	return true
end


-- 88  dP 888888 Yb  dP  dP""b8    db    88""Yb 888888 88   88 88""Yb 888888
-- 88odP  88__    YbdP  dP   `"   dPYb   88__dP   88   88   88 88__dP 88__
-- 88"Yb  88""     8P   Yb       dP__Yb  88"""    88   Y8   8P 88"Yb  88""
-- 88  Yb 888888  dP     YboodP dP""""Yb 88       88   `YbodP' 88  Yb 888888
guiElement.keycapture = {}
function guiElement.keycapture:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v)
	return _i
end
function guiElement.keycapture:setup(v)
	self.title = v.title
	self.func = v.func
	self.init = v.init
	self.type = v.type
	self.value = v.value or ""
	self.maxlength = v.maxlength or 20
	self.desc = v.desc
	self.padding = v.padding
	self.selAnimTime = 0
	self.selAnimSpeed = v.selAnimSpeed
	self.selAnimTo = 0
	self.width = v.parent.width--font["menu"]:getWidth(self.title)
	self.height = getFontHeight("menu")
	self.boxWidth = 100
	self.boxHeight = getFontHeight("menu")
	self.chars = v.chars
	self.mode = 1
	self.parent = v.parent
	self.enabled = v.enabled
	self.textColor = v.textColor
end
function guiElement.keycapture:update(dt)
	self.caret = "??"
	self.caretLeft = getFontWidth("menu", self.value)
	if math.cos(love.timer.getTime() * 10) >= 0 then self.caret = "" end
end
function guiElement.keycapture:draw(parent, x, y, shadow, focused)
	lgr.setLineWidth(2)
	shadow = shadow or false
	lgr.printf(self.title, x, y, parent.width, "left")
	local r,g,b,a = lgr.getColor()

	lgr.push()
	lgr.translate(0,1.5)
	if not shadow then
		lgr.setColor(0,0,0,50)
		lgr.rectangle("fill", x+(parent.width-self.boxWidth)-4, y-2+1, self.boxWidth+8, self.boxHeight+2+1)
		local a = 0.25
		if focused then a = 1 end
		lgr.setColor(255,255,255,255*a)
		lgr.rectangle("line", x+(parent.width-self.boxWidth)-4, y-2+1, self.boxWidth+8, self.boxHeight+2+1)
	end

	-- lgr.setScissor(parent.x + x+(parent.width-self.boxWidth), parent.y + y, self.boxWidth, self.boxHeight)

	lgr.setColor(r,g,b,a)
	setFont("menu")
	if self.mode == 1 then
	    local chars = { [" "] = "SPACE", ["lshift"] = "L Shift", ["rshift"] = "R Shift", ["return"] = "ENTER", ["capslock"] = "CAPS" }
	    local txt = (chars[self.value] or self.value)
	    if txt == "" then txt = "--" end
		lgr.printf(string.upper(txt), x+(parent.width-self.boxWidth), y, self.boxWidth, "center")
	elseif self.mode == 2 then
	    lgr.printf(string.upper(self.caret), x+(parent.width-self.boxWidth), y, self.boxWidth, "center")
	end

	-- lgr.setScissor()
	lgr.pop()
end
function guiElement.keycapture:keypressed(k, u)
	local allow
	if self.mode == 1 then
		if k == " " or k == "return" or k == "enter" then
			self.mode = 2
		elseif k == "escape" then
			self.parent:back()
		end
		allow = true
	elseif self.mode == 2 then
		if k == "escape" then
			self.mode = 1
		else
			self.value = k
			self.mode = 1
			self.func(self.value)
		end
		allow = false
	end
	return allow
end



-- .dP"Y8 888888 88""Yb    db    88""Yb    db    888888  dP"Yb  88""Yb
-- `Ybo." 88__   88__dP   dPYb   88__dP   dPYb     88   dP   Yb 88__dP
-- o.`Y8b 88""   88"""   dP__Yb  88"Yb   dP__Yb    88   Yb   dP 88"Yb
-- 8bodP' 888888 88     dP""""Yb 88  Yb dP""""Yb   88    YbodP  88  Yb
guiElement.separator = {}
function guiElement.separator:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v)
	return _i
end
function guiElement.separator:setup(v)
	self.type = v.type
	self.width = 0
	self.height = 1
	self.parent = v.parent
	self.textColor = v.textColor
end
function guiElement.separator:update(dt)

end
function guiElement.separator:draw(parent, x, y, shadow, focused)
	lgr.setLineWidth(2)
	if not shadow then
		lgr.setColor(color.Silver)
	end
	lgr.line(x + 10, y+2.5, parent.width - 10, y+2.5)
end
