local lgr = love.graphics
local tmr = love.timer

level = {}

function level:load(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v)
	return _i
end
function level:setup(v)
	game.skill = v
	local s = 0.2
	self.name = "Level " .. game.skill
	self.timeUntilNextFood = 1
	self.bounds = {
		offsetX = 0,
		offsetY = 100,
		width = DRAW_WIDTH * s,
		height = (DRAW_HEIGHT - 120) * s,
		scale = 1,
		leftEdge = 100
	}
	self.bounds.leftEdge = self.bounds.width * 0.20
	self.bounds.scale = DRAW_WIDTH / self.bounds.width
	self.totalFood = 0
	self.foodOnScreen = 0
	self.totalfloaters = 0
	self.finished = false

	self.waitTimer = 1

	self.track = {
		[1] = {active = false, countdown = math.random(0,2) * 0.5},
		[2] = {active = false, countdown = math.random(0,2) * 0.5},
		[3] = {active = false, countdown = math.random(0,2) * 0.5},
	}

	local lsk = levelStats[game.skill] or levelStats[#levelStats]

	self.speedList = lsk.speedList
	self.foodNeededForNext = lsk.foodNeededForNext
end

function level:update(dt)
	self.waitTimer = math.max(self.waitTimer - dt, 0)
	if self.waitTimer == 0 then
		-- self.timeUntilNextFood = self.timeUntilNextFood - dt
		for i = 1, #self.track do
			self.track[i].countdown = self.track[i].countdown - dt
			if self.track[i].active == false and self.track[i].countdown <= 0 then
				spawnFood(i)
				self.track[i].active = true
				self.track[i].countdown = 0
			end
		end

		if game.playmode == 2 then
			game.countdownTimer = math.max(game.countdownTimer - dt, 0)
			if game.countdownTimer <= 0 then
				changeState("gameover")
			end
		end
	end

	-- if self.timeUntilNextFood <= 0 then
	-- 	spawnFood()
	-- 	self.timeUntilNextFood = math.random(100,1000) / 1000
	-- end
end

function level:draw()
	lgr.push()
	lgr.translate(80, 5)
	printShadowed {text = "Calories:", font = "status", width = 440, align = "left"}
	printShadowed {text = string.formatNumber(game.player.calories), font = "statusbold", width = 440, align = "right"}
	lgr.pop()

	lgr.push()
	lgr.translate(DRAW_WIDTH - (440 + 80), 5)
	printShadowed {text = "Score:", font = "status", width = 440, align = "left"}
	printShadowed {text = string.formatNumber(game.player.points), font = "statusbold", width = 440, align = "right"}

	if multiplierList[game.multiplier] > 1 then
		lgr.translate(0, 60)
		printShadowed {text = "x" .. (multiplierList[game.multiplier] * game.skill), font = "multiplier", width = 140, align = "right"}
	end
	lgr.pop()

	if game.playmode == 2 then
		lgr.push()
		lgr.translate(800, 5)
		printShadowed {text = "Time:", font = "status", width = 330, align = "left"}
		printShadowed {text = string.formatTime(game.countdownTimer, true), font = "statusbold", width = 330, align = "right"}
		lgr.pop()
	end

	-- lgr.setFont(font["smallstat"])
	-- lgr.print(game.level.foodNeededForNext - game.level.totalFood, DRAW_WIDTH - 80, DRAW_HEIGHT - 50)
	-- lgr.print(game.level.foodOnScreen .. "\n" .. game.totalFoodSpawns, DRAW_WIDTH - 80, DRAW_HEIGHT - 130)

	if game.playmode == 1 then
		lgr.push()
		lgr.translate(800, 0)
		lgr.push()
		lgr.scale(1)
		lgr.translate(0,50)
		for i = 1, game.player.rottenPickles do
			local r = math.sin(tmr.getTime() + (2000 * i)) * 0.1
			lgr.setColor(0,0,0,100)
			lgr.draw(game.pickleIcon, (40 * spriteScale) / 2 * (i-1), 4, r, 1, 1, 40, 40)
			lgr.setColor(255,255,255)
			lgr.draw(game.pickleIcon, (40 * spriteScale) / 2 * (i-1), 0, r, 1, 1, 40, 40)
		end
		if game.player.rottenPickles < 5 then
			for i = game.player.rottenPickles+1, 5 do
				local r = math.sin(tmr.getTime() + (2000 * i)) * 0.1
				lgr.setColor(0,0,0,50)
				lgr.draw(game.pickleIcon, (40 * spriteScale) / 2 * (i-1), 4, r, 1, 1, 40, 40)
			end
		end
		lgr.pop()
		lgr.pop()
	end

	lgr.setLineWidth(2)
	lgr.setColor(255,255,255)
	lgr.line(0, 100, DRAW_WIDTH, 100)
	lgr.setColor(0,0,0,100)
	lgr.line(0, 102, DRAW_WIDTH, 102)
end

levelStats = {
	[1] = { speedList = {1,1,1,2,2}, foodNeededForNext = 50 },
	[2] = { speedList = {2,2,2,2,3}, foodNeededForNext = 50 },
	[3] = { speedList = {2,3,3,3,4}, foodNeededForNext = 50 },
	[4] = { speedList = {3,3,4,4,4}, foodNeededForNext = 50 },
	[5] = { speedList = {4,4,4,5,5}, foodNeededForNext = 50 },
	[6] = { speedList = {5,5,5,5,6}, foodNeededForNext = 50 },
	[7] = { speedList = {6,6,6,7,8}, foodNeededForNext = 50 },
	[8] = { speedList = {7,8,7,8,7}, foodNeededForNext = 50 },
}
