local lgr = love.graphics
local kbd = love.keyboard
local tmr = love.timer

require 'fonts'
require 'funct'
require 'level'
require 'player'
require 'food'
require 'floatpoints'
require 'sound'
require 'message'
require 'background'
require 'gui'
require 'timer'
require 'transition'
require 'colors'
easing = require 'easing'

game = {}

local draw = lgr.drawq
if isNine() then draw = lgr.draw end

_dbg = { on = false}

spriteScale = 4
spriteSize = 200

DRAW_WIDTH = 1920
DRAW_HEIGHT = 1080

if isNine() then
	screenModes = love.window.getModes()
else
	local screenModes2 = lgr.getModes()
	screenModes = {
		[1] = screenModes2[1],
		-- [2] = { width = 1920, height = 1080 },
		[2] = { width = 1280, height = 720 },
		[3] = { width = 1024, height = 640 },
	}
end
currentScreenMode = #screenModes
for i, m in pairs(screenModes) do
	print(i, m.width, m.height)
	if m.width == 1024 then currentScreenMode = i end
end

function love.load()
	-- joystick = 1

	-- if not isNine then
	-- 	require 'xboxlove'
	-- 	controller = xboxlove.create(joystick)
	-- 	deadzone = 0
	-- 	if controller then
	-- 		controller:setDeadzone("ALL",deadzone)
	-- 	end
	-- end

	sfx = {}
	sfx["intro"] = love.sound.newSoundData("audio/intro.ogg")
	sfx["spawnfood"] = love.sound.newSoundData("audio/spawnfood.ogg")
	sfx["eatfood"] = love.sound.newSoundData("audio/eatfood.ogg")
	sfx["eatpickle"] = love.sound.newSoundData("audio/eatpickle.ogg")
	sfx["levelup"] = love.sound.newSoundData("audio/levelup.ogg")
	sfx["gameover"] = love.sound.newSoundData("audio/gameover.ogg")

	loadPreferences()

	game.totalFoodSpawns = 0
	game.screen = {
		width = 640, height = 350, fullscreen = false
	}
	game.screen.upScale = 1--DRAW_WIDTH / game.screen.width
	game.state = "menu"
	game.playmode = 1
	game.zoom = 0
	game.zoomTo = 0

	local fnt = "menu"
	local th = getFontHeight(fnt) * 1.12
	local pad = 4
	local ali = "center"
	game.menu = guiSystem:create({ width = 300, snap = {"center", "bottom"}, margin = {60, 20, 60, 100, "fill", "fit"}, bgcolor = {0,0,0,0} })
	game.menu:addPage("main", {font = fnt, spacing = th, padding = pad, align = ali, changeTo = function() end})
	game.menu:addListItem("main", { title = "Play Game", func = function() game.menu:setCurrentPage("game_mode") end })
	game.menu:addListItem("main", { title = "How To Play", func = function()
		transition.start("in", "fade", .2,
			function()
				newTimer(.1, function()
					changeState("howtoplay")
					transition.start("out", "fade", .2, nil, {32,180,24})
				end)
			end, {32,180,24}
		)
	end })
	game.menu:addListItem("main", { title = "Options", func = function() game.menu:setCurrentPage("options") end })
	game.menu:addListItem("main", { title = "Quit Game", func = function() game.menu:setCurrentPage("quit_confirm") end })

	--Game Mode
	game.menu:addPage("game_mode", {font = fnt, header = "Game Mode", spacing = th, padding = pad, align = ali, width = 300})
	game.menu:addListItem("game_mode", { title = "Classic", func = function()
		transition.start("in", "fade", .2,
			function()
				newTimer(.2, function()
					game.playmode = 1
					changeState("game")
					transition.start("out", "fade", .2, nil, color.Black)
				end)
			end, color.Black
		)
	end })
	game.menu:addListItem("game_mode", { title = "3 Minute Rule", func = function()
		transition.start("in", "fade", .2,
			function()
				newTimer(.2, function()
					game.playmode = 2
					changeState("game")
					transition.start("out", "fade", .2, nil, color.Black)
				end)
			end, color.Black
		)
	end })
	game.menu:addListItem("game_mode", { title = "Back", func = function() game.menu:back() end, desc = "Return to Options menu" })

	--Game Options
	game.menu:addPage("options", {font = fnt, spacing = th, padding = pad, align = ali, width = 500})
	game.menu:addListItem("options", { title = "Toggle Fullscreen (F11)", func = function() setFullscreen(not game.screen.fullscreen) end })
	game.menu:addListItem("options", { title = "Sound Effects", type = "checkbox", appearance = "ONOFF", enabled = true,
		func = function(v)
			preferences.soundOn = v
			addMessage("Sound effects are now "..string.upper(onOff(preferences.soundOn)))
			playSound("spawnfood")
		end, init = function() return preferences.soundOn end
	})
	game.menu:addListItem("options", { title = "Configure Controls", func = function() game.menu:setCurrentPage("control_options") end })
	game.menu:addListItem("options", { title = "Back", func = function() game.menu:back() end, desc = "Return to Main menu" })

	local control_desc = "Press Enter then press the key you wish to use - Press Escape to cancel"
	game.menu:addPage("control_options", {font = fnt, spacing = th, padding = pad, align = ali, width = 500})
	game.menu:addListItem("control_options", { title = "Move Left", type = "keyCapture", func = function() end, desc = control_desc,
		func = function(v) preferences.key_left = v end, init = function() return preferences.key_left end })
	game.menu:addListItem("control_options", { title = "Right", type = "keyCapture", func = function() end, desc = control_desc,
		func = function(v) preferences.key_right = v end, init = function() return preferences.key_right end })
	game.menu:addListItem("control_options", { title = "Up", type = "keyCapture", func = function() end, desc = control_desc,
		func = function(v) preferences.key_up = v end, init = function() return preferences.key_up end })
	game.menu:addListItem("control_options", { title = "Down", type = "keyCapture", func = function() end, desc = control_desc,
		func = function(v) preferences.key_down = v end, init = function() return preferences.key_down end })
	game.menu:addListItem("control_options", { title = "Back", func = function() game.menu:back() end, desc = "Return to Options menu" })


	--Quit Confirm
	game.menu:addPage("quit_confirm", {font = fnt, header = "You Sure?", spacing = th, padding = pad, align = ali, width = 400})
	game.menu:addListItem("quit_confirm", { title = "Continue", func = function() game.menu:back() end })
	game.menu:addListItem("quit_confirm", { title = "Quit Game", func = function()
			game.menu:setCurrentPage()
			game.menu:clearSelection()
			newTimer(.2, function()
				transition.start("in", "fade", .2,
					function()
						newTimer(.2, function() love.event.quit() end)
					end, color.Clouds)
				end
			)
		end
	})

	game.menu:setCurrentPage("main", true)



	--Pause Menu
	game.pauseMenu = guiSystem:create({ width = 300, snap = {"center", "bottom"}, margin = {60, 20, 60, 240, "fill", "fit"}, bgcolor = {0,0,0,0} })
	game.pauseMenu:addPage("pause_menu", {font = fnt, spacing = th, padding = pad, align = ali, width = 300})
	game.pauseMenu:addListItem("pause_menu", { title = "Continue",
		func = function()
			-- game.menu:setCurrentPage()
			game.pauseMenu:clearSelection()
			game.paused = false
			game.zoomTo = 0
		end
	})
	game.pauseMenu:addListItem("pause_menu", { title = "Quit to Menu", func = function() game.pauseMenu:setCurrentPage("quit_confirm") end })

	--Quit Confirm
	game.pauseMenu:addPage("quit_confirm", {font = fnt, header = "You Sure?", spacing = th, padding = pad, align = ali, width = 400})
	game.pauseMenu:addListItem("quit_confirm", { title = "Continue Playing", func = function() game.pauseMenu:back() end })
	game.pauseMenu:addListItem("quit_confirm", { title = "Quit Game", func = function()
			game.pauseMenu:setCurrentPage()
			game.pauseMenu:clearSelection()
			game.zoomTo = 0
			transition.start("in", "toleft", 1,
				function()
					game.pauseMenu:setCurrentPage("pause_menu", true)
					newTimer(.5, function()
						changeState("menu")
						transition.start("out", "toright", 1, nil, color.Black)
					end)
				end, color.Black
			)
		end
	})

	game.pauseMenu:setCurrentPage("pause_menu", true)



	game.foodSheetLinear = lgr.newImage("images/food_x"..spriteScale..".png")
	game.foodSheet = lgr.newImage("images/food_x"..spriteScale..".png")
	game.foodSheet:setFilter("nearest", "linear")
	game.foodQuad = {}
	local w, h = game.foodSheet:getWidth(), game.foodSheet:getHeight()
	for y = 1, h / spriteSize do
		for x = 1, w / spriteSize do
			game.foodQuad[#game.foodQuad+1] = lgr.newQuad((x - 1) * spriteSize, (y - 1) * spriteSize, spriteSize, spriteSize, w, h)
		end
	end

	game.charSheet = lgr.newImage("images/char_x"..spriteScale..".png")
	game.charSheet:setFilter("nearest", "linear")
	game.charQuad = {}
	local w, h = game.charSheet:getWidth(), game.charSheet:getHeight()
	for y = 1, h / spriteSize do
		for x = 1, w / spriteSize do
			game.charQuad[#game.charQuad+1] = lgr.newQuad((x - 1) * spriteSize, (y - 1) * spriteSize, spriteSize, spriteSize, w, h)
		end
	end

	game.pickleIcon = lgr.newImage("images/pickle_icon.png")

	changeState("menu")

	changeScreenMode(2)

	lgr.setLineWidth(4)

	playSound("intro")
	lgr.setBackgroundColor(0,0,0)

	createScrollingFoodBackground()
	updateWindowSize()
end

function love.update(dt)
	dt = math.min(dt, 1/30)

	updateTimers(dt)
	transition.update(dt)

	if game.state == "menu" then
		game.menu:update(dt)

	elseif game.state == "howtoplay" then
		game.howToTimer = game.howToTimer + dt
	elseif game.state == "game" then
		if game.zoom < game.zoomTo then game.zoom = math.min(game.zoom + dt * 10, 1) end
		if game.zoom > game.zoomTo then game.zoom = math.max(game.zoom - dt * 15, 0) end

		if game.focused == false and game.paused == false then game.paused = true game.zoomTo = 1 end
		game.deadFoodList = {}
		game.deadFloaterList = {}

		if not game.paused then
			game.level:update(dt)

			for i, f in pairs(game.food) do
				f:update(dt)
			end
			for i, f in pairs(game.floaters) do
				f:update(dt)
			end

			game.player:update(dt)
		else
			game.pauseMenu:update(dt)
		end

		destroyFood()
		destroyFloaters()

		if game.totalFoodSpawns >= game.level.foodNeededForNext and game.level.foodOnScreen <= 0 and game.player.animationFinished then
			game.level.finished = true
		end

		if game.level.finished then
			print("Level up!")
			changeState("levelup")
		end
	elseif game.state == "levelup" then
		game.intermissionTimer = game.intermissionTimer - dt
		if game.intermissionTimer <= 0 then changeState("game") end
	elseif game.state == "gameover" then
		game.intermissionTimer = game.intermissionTimer - dt
		if game.intermissionTimer <= 0 then changeState("menu") end
	end

	if isNine() then
		love.window.setCaption("Eat It! " .. tmr.getFPS())
	else
		lgr.setCaption("Eat It! " .. tmr.getFPS())
	end

	updateMessageSystem(dt)
end

function love.draw()
	local z
	if game.state == "game" then z = 1 - (game.zoom * 0.15) else z = 1 end
	local x, y, w, h =	game.screen.windowScale.padX + ((DRAW_WIDTH * game.screen.windowScale.w) * (1-z) / 2),
						game.screen.windowScale.padY + ((DRAW_HEIGHT * game.screen.windowScale.h) * (1-z) / 2),
						DRAW_WIDTH * game.screen.windowScale.w * z,
						DRAW_HEIGHT * game.screen.windowScale.h * z
	lgr.setScissor( x, y, w, h )

	lgr.push()
	lgr.translate(game.screen.windowScale.padX, game.screen.windowScale.padY)
	lgr.scale(game.screen.windowScale.w, game.screen.windowScale.h)

	if game.state == "menu" then
		drawBackground(205,81,117)

		drawScrollingFoodBackground()

		lgr.push()
		lgr.scale(1)

		lgr.push()
		lgr.translate(DRAW_WIDTH/2, 60)
		lgr.push()
		lgr.shear(math.sin(tmr.getTime()*0.5)/10,math.cos(tmr.getTime())/14)
		lgr.setColor(0,0,0,100)
		lgr.rectangle("fill", -(DRAW_WIDTH/2+100), -6+80, DRAW_WIDTH+200, 540)
		lgr.setColor(53,150,200)
		lgr.rectangle("fill", -(DRAW_WIDTH/2+100), 80, DRAW_WIDTH+200, 540)
		printShadowed {text = "Eat It!", font = "title", width = DRAW_WIDTH, align = "center", shadowOffset = -6, x = -(DRAW_WIDTH/2), y = 0}
		lgr.pop()
		lgr.pop()

		-- local tt = "Press S to change window size / "
		-- if isNine then tt = "" end
		-- printShadowed {text = "Press Enter to Pig Out!", font = "menu", width = DRAW_WIDTH, align = "center", shadowOffset = -3,
		-- 	x = math.sin((tmr.getTime()*5))*10, y = 750}
		-- printShadowed {text = "Press H for How to Play", font = "menu", width = DRAW_WIDTH, align = "center", shadowOffset = -3,
		-- 	x = math.sin((tmr.getTime()+1000)*5)*10, y = 810}
		-- printShadowed {text = tt.."F11 for fullscreen", font = "menu", width = DRAW_WIDTH, align = "center", shadowOffset = -3,
		-- 	x = math.sin((tmr.getTime()+2000)*5)*10, y = 870}
		-- printShadowed {text = "Press A to turn sound effects " .. string.upper(onOff(not preferences.soundOn)), font = "menu",
		-- 	width = DRAW_WIDTH, align = "center", shadowOffset = -3, x = 0+math.sin((tmr.getTime()+3000)*5)*10, y = 930}


		printShadowed {text = "A loving tribute to \"Fast Food\", © 1982 Telesys by Don Ruffcorn - Recreated and modernized by Rogue Carrot 2013", font = "multiplier", width = DRAW_WIDTH, align = "center", shadowOffset = -4, x = 0, y = 1030}

		printShadowed {text = "High Score: " .. string.formatNumber(highScoreTable[1].score), font = "menu", width = 400, align = "right",
			shadowOffset = -4, x = DRAW_WIDTH - 1000, y = 20}
		printShadowed {text = "Calories: " .. string.formatNumber(highScoreTable[1].calories), font = "multiplier", width = 400, align = "right",
			shadowOffset = -4, x = DRAW_WIDTH - 1000, y = 70}
		printShadowed {text = "Level: " .. string.formatNumber(highScoreTable[1].level), font = "multiplier", width = 400, align = "right",
			shadowOffset = -4, x = DRAW_WIDTH - 1000, y = 110}

		printShadowed {text = "High Score: " .. string.formatNumber(highScoreTable[2].score), font = "menu", width = 400, align = "right",
			shadowOffset = -4, x = DRAW_WIDTH - 470, y = 20}
		printShadowed {text = "Calories: " .. string.formatNumber(highScoreTable[2].calories), font = "multiplier", width = 400, align = "right",
			shadowOffset = -4, x = DRAW_WIDTH - 470, y = 70}
		printShadowed {text = "Level: " .. string.formatNumber(highScoreTable[2].level), font = "multiplier", width = 400, align = "right",
			shadowOffset = -4, x = DRAW_WIDTH - 470, y = 110}

		lgr.pop()

		game.menu:draw(0,0)

	elseif game.state == "howtoplay" then
		drawBackground(32,180,24)

		drawScrollingFoodBackground()

		lgr.push()
		lgr.translate(170, 60)
		local n = 1
		local s = 0.85
		local spacing = 100 * s
		printShadowed {text = "Food:", font = "smallstat", width = 500, align = "left", shadowOffset = -4, x = 140, y = -40}
		printShadowed {text = "Calories:", font = "smallstat", width = 100, align = "center", shadowOffset = -4, x = 440, y = -40}
		for i, f in pairs(helpFoodList) do
			lgr.push()
			lgr.translate(math.sin(tmr.getTime() * 3 + (2000 * n)) * 10 + (n * 10), math.sin(tmr.getTime() * 3 + (6000 * n)) * 2)
			printShadowed {text = f.name, font = "foodlist", width = 500, align = "left", shadowOffset = -4, x = 140, y = spacing * (n-1) + 10}
			printShadowed {text = f.calories, font = "foodlistbold", width = 100, align = "center", shadowOffset = -4, x = 440, y = spacing * (n-1) + 10}

			local r = math.sin(tmr.getTime() * 3 + (2000 * n)) * 0.1
			lgr.setColor(0,0,0,100)
			draw(game.foodSheet, game.foodQuad[f.quad],
				20 + spacing / 2, spacing * (n-1) + spacing / 2 - 4, r, s * 1.1 / 2, s * 1.1 / 2, (spriteSize / 2), (spriteSize / 2))

			lgr.setColor(255,255,255)
			draw(game.foodSheet, game.foodQuad[f.quad],
				20 + spacing / 2, spacing * (n-1) + spacing / 2, r, s * 1.1 / 2, s * 1.1 / 2, (spriteSize / 2), (spriteSize / 2))
			lgr.pop()

			n = n + 1
		end
		lgr.pop()

		local howToText = "HOW TO PLAY:\nI hope you're hungry because you're in for the feast of your life! Eat as much as you can and rack up those calories! But watch out for the PURPLE PICKLES! They will stick around and eventually give you a bad case of HEARTBURN! Once you get six of those it's all over.\n\nEat as much food as you can. Each type of food has its own calorie count so try and go for the most. Every 500 calories one purple pickle will be removed from your total, but it's not as easy as it sounds. Every once in a while you will need to take a break and when it's time to return, the food will be even faster than before!\n\nThink you can keep up?\n\nSCORING:\nEvery piece of food is worth 10 points x the current multiplier x the current level. The multiplier is determined by how many subsequent pieces of food you can eat without hitting a purple pickle."
		printShadowed {text = string.sub(howToText, 0, game.howToTimer * 800), font = "smallstat", width = 800, align = "left", shadowOffset = -4, x = 1000, y = 40}
	elseif game.state == "game" then
		lgr.push()
		lgr.translate(DRAW_WIDTH / 2, DRAW_HEIGHT / 2)

		lgr.push()
		lgr.scale(z)
		lgr.translate((-DRAW_WIDTH / 2), (-DRAW_HEIGHT / 2))

		drawBackground(53,150,200)
		lgr.push()
		lgr.scale(game.screen.upScale)

		lgr.setFont(font["status"])
		local glb = game.level.bounds
		-- local ws, hs = game.screen.width/glb.width, game.screen.height/glb.height

		lgr.push()
		lgr.translate(glb.offsetX, glb.offsetY)
		lgr.push()
		lgr.scale(glb.scale)

		-- lgr.setColor(0,255,0, 100)
		-- lgr.rectangle("fill", 0, 0, glb.width, glb.height)

		lgr.setColor(255,255,255)
		for i, f in pairs(game.food) do
			f:draw()
		end

		for i, f in pairs(game.floaters) do
			f:draw()
		end

		game.player:draw()

		lgr.pop()
		lgr.pop()

		-- lgr.setScissor()

		lgr.pop()

		lgr.push()
		lgr.scale(4)
		lgr.setColor(255,255,255)
		lgr.setFont(font["dialog"])
		-- lgr.print(glb.width .. ", " .. glb.height .. "      " .. game.level.timeUntilNextFood, 200, 0)
		-- lgr.print(game.screen.windowScale.w .. ", " .. game.screen.windowScale.h, 0, 90)
		-- lgr.print(game.totalFoodSpawns .. ", " .. game.totalFoodRemovals, 0, 100)
		-- lgr.print(tmr.getFPS(), 1, 1)

		lgr.pop()
		-- for i = 1, #game.level.speedList do
		-- 	lgr.print(game.level.speedList[i], i * 10, 30)
		-- end

		game.level:draw()

		lgr.pop()
		lgr.pop()
		lgr.setScissor()

		if game.paused then
			lgr.setColor(0,0,0,100)
			lgr.rectangle("fill", 0, 0, DRAW_WIDTH, DRAW_HEIGHT)
			printShadowed {text = "PAUSED", font = "pause", width = DRAW_WIDTH, align = "center", shadowOffset = -4, x = 0, y = 320, color = {255,255,255,255*game.zoom}, shadowColor = {0,0,0,100 * game.zoom}}
			game.pauseMenu:draw()
		end

		-- local y = 0
		-- lgr.setFont(font["dialog"])
		-- lgr.print("Foodlist:", 10, 40)
		-- for i, f in pairs(game.food) do
		-- 	lgr.print(f.id, 13, 50 + y * 10)
		-- 	y = y + 1
		-- end

	elseif game.state == "levelup" then
		drawBackground(220,150,53)

		-- lgr.push()
		-- lgr.scale(game.screen.upScale)
		-- local glb = game.level.bounds
		-- local ws, hs = 640/glb.width, 350/glb.height
		-- lgr.push()
		-- lgr.translate(glb.offsetX, glb.offsetY)
		-- lgr.scale(ws, hs)

		-- lgr.setColor(255,255,255)
		-- game.player:draw()

		-- lgr.pop()
		-- lgr.pop()

		local scale = 16
		local y = 100 / scale
		-- lgr.push()
		-- lgr.scale(scale)
		-- lgr.setFont(font["dialog"])
		-- lgr.setColor(255,255,255)
		-- lgr.printf("You're\nGetting\nFatter!", 0, y, DRAW_WIDTH / scale, "center")
		lgr.push()
		lgr.translate(DRAW_WIDTH / 2, DRAW_HEIGHT / 2)
		lgr.rotate((-game.intermissionTimer + 1) / 5)
		lgr.scale(game.intermissionTimer)
		local t = "You're\nGetting\nFatter!"
		local w, l = font["fatter"]:getWrap(t, DRAW_WIDTH)
		printShadowed {
			text = t, font = "fatter", width = DRAW_WIDTH, align = "center",
			shadowOffset = -4, x = -(DRAW_WIDTH / 2), y = -((font["fatter"]:getHeight() * l) / 2)}
		lgr.pop()

		-- lgr.pop()
		-- lgr.print(game.intermissionTimer .. "  " .. y, 0, 0)
		-- for i = 1, #game.player.foodCollected do
		-- 	local n = foodStats[game.player.foodCollected[i]].quad
		-- 	local s = 1 / spriteScale -- + (math.cos(self.rotation * 2) / 10)
		-- 	lgr.setColor(255,255,255)
		-- 	draw(game.foodSheet, game.foodQuad[n], i * ((25 * spriteScale) * s), ((42 * spriteScale) * s), 0, s, s, (spriteSize / 2), (spriteSize / 2))
		-- end
	elseif game.state == "gameover" then
		drawBackground(220,50,53)
		lgr.push()
		lgr.translate(DRAW_WIDTH / 2, DRAW_HEIGHT / 2)
		lgr.rotate((-game.intermissionTimer + 1) / 5)
		lgr.scale(game.intermissionTimer)
		local scale = 16
		local y = ((game.intermissionTimer / 3) * DRAW_HEIGHT) / scale
		-- local y = 100 / scale
		local t = "BURRRRRRP!"
		local w, l = font["fatter"]:getWrap(t, DRAW_WIDTH)
		printShadowed {
			text = t, font = "fatter", width = DRAW_WIDTH, align = "center",
			shadowOffset = -4, x = -(DRAW_WIDTH / 2) + math.random(-10,10), y = -((font["fatter"]:getHeight() * l) / 2) + math.random(-10,10)}
		lgr.pop()
	end
	lgr.pop()


	-- lgr.setColor(0,0,0)
	-- lgr.rectangle("fill", 0, 0, lgr.getWidth(), game.screen.windowScale.padY)
	-- lgr.rectangle("fill", 0, lgr.getHeight() - game.screen.windowScale.padY, lgr.getWidth(), game.screen.windowScale.padY)

	transition.draw()

	lgr.push()
	lgr.translate(game.screen.windowScale.padX, game.screen.windowScale.padY)
	drawMessageSystem()
	lgr.pop()


	-- lgr.push()
	-- lgr.scale(10)
	-- lgr.setColor(255,255,255)
	-- draw(game.foodSheet, game.foodQuad[(math.floor(tmr.getTime()) % 11) + 1], 0, 0, 0, .1, .1)
	-- lgr.pop()
end

function love.keypressed(k, u)
	if k == "s" then
		changeScreenMode(kbd.isDown("lshift", "rshift"))
	elseif k == "a" then
		preferences.soundOn = not preferences.soundOn
		addMessage("Sound effects are now "..string.upper(onOff(preferences.soundOn)))
		playSound("spawnfood")
	elseif k == "f11" then
		setFullscreen(not game.screen.fullscreen)
	end

	if game.state == "menu" then
		game.menu:keypressed(k, u)
	elseif game.state == "howtoplay" then
		if k == "return" or k == "escape" or k == " " then
			transition.start("in", "fade", .2,
				function()
					newTimer(.1, function()
						changeState("menu")
						transition.start("out", "fade", .2, nil, {205,81,117})
					end)
				end, {205,81,117}
			)
		end
	elseif game.state == "game" then
		if k == "p" or k == "escape" then
			game.paused = not game.paused
			if game.paused then game.zoomTo = 1 else game.zoomTo = 0 end
		elseif k == "0" then
			game.player.rottenPickles = 5
		elseif k == "1" then
			game.level.foodNeededForNext = 1
		end

		if game.paused then
			game.pauseMenu:keypressed(k, u)
		end

		if kbd.isDown("lmeta") then
			if k == "w" then
				print("Close window")
			end
		end
	end
end

function love.focus(f)
	game.focused = f
end

function love.quit()
	savePreferences()
end

function love.resize(w, h)
	print("Window resized to ", w, "x", h)
	currentScreenMode = 0
	updateWindowSize(false)
	addMessage("Resized window to " .. w .. "x" .. h)
end
