local lgr = love.graphics

messageSystem = {}
local totalMessages = 0
local messagesToRemove = {}

function updateMessageSystem(dt)
	for i, m in pairs(messagesToRemove) do
		messageSystem[i] = nil
		messagesToRemove[i] = nil
		-- print("Removed message")
	end

	for i, m in pairs(messageSystem) do
		m:update(dt)
	end
end

function drawMessageSystem()
	lgr.push()
	lgr.scale(2)
	local y = 5
	for i, m in pairs(messageSystem) do
		m:draw(5, y)
		y = y + m.height
	end
	lgr.pop()
end

function addMessage(text)
	totalMessages = totalMessages + 1
	messageSystem[totalMessages] = message:create {text = text, id = totalMessages}
end


message = {}

function message:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v)
	return _i
end
function message:setup(v)
	v = v or {}
	self.text = v.text or ""
	self.life = 2
	self.id = v.id
	self.fade = 1
end


function message:update(dt)
	self.life = self.life - dt
	if self.life <= 0 then
		self.fade = self.fade - (dt * 4)
		if self.fade <= 0 then
			self.fade = 0
			messagesToRemove[self.id] = true
		end
	end
	self.height = 10 * self.fade
end

function message:draw(x, y)
	local c = {255,255,255,255 * self.fade}
	local sc = {0,0,0,100 * self.fade}
	printShadowed {text = self.text, font = "cga", width = DRAW_WIDTH, align = "left", shadowOffset = 1, x = x, y = y, color = c, shadowColor = sc}
end
