local lgr = love.graphics
local kbd = love.keyboard
local tmr = love.timer

local draw = lgr.drawq
if love.window then draw = lgr.draw end

playerObject = {}
function playerObject:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v)
	return _i
end
function playerObject:setup(v)
	v = v or {}
	self.x = v.x or 0
	self.y = v.y or 0
	self.speed = (v.speed or 320)
	self.vspeed = self.speed * 0.65
	self.rad = 23

	self.spriteSheet = game.charSheet
	self.quadList = game.charQuad
	self.animationFrameList = {4,4,4,3,2,1,1,2,3,4,5}
	self.animationSpeed = 20
	self.animationFinished = false

	self.rottenPickles = 0

	self.points = 0
	self.calories = 0

	self.foodCollected = {}

	self.calorieCounter = 0

	self.facing = 1
	self.spin = 1
	self.turnSpeed = 20

	self.frame = 1

	self.anticipation = false
	self.anticipateFood = 0
	self.foodSincePickle = 0
	self.bob = 0
	self.weave = 0
end

function playerObject:update(dt)
	self.animationFinished = false
	self.bob = math.sin(tmr.getTime() * 4) * 1
	self.weave = math.sin(tmr.getTime() * 3) * 0.05
	local oldFrame = math.floor(self.frame)
	self.frame = math.max(1, self.frame - dt * self.animationSpeed)
	if math.floor(self.frame) == 1 then
		self.animationFinished = true
	end

	if kbd.isDown(preferences.key_left) then
		self.x = self.x - dt * self.speed
		-- if not kbd.isDown("lshift", "rshift") then
		self.spin = 1
		-- end
	elseif kbd.isDown(preferences.key_right) then
		self.x = self.x + dt * self.speed
		self.spin = -1
	else
		self.spin = 1
	end

	if kbd.isDown(preferences.key_up) then
		self.y = self.y - dt * self.vspeed
		self.rotation = (0.1) * self.facing
	elseif kbd.isDown(preferences.key_down) then
		self.y = self.y + dt * self.vspeed
		self.rotation = (-0.1) * self.facing
	else
		self.rotation = 0
	end

	if self.x < (game.level.bounds.leftEdge + 25) then self.x = (game.level.bounds.leftEdge + 25) end
	if self.x > game.level.bounds.width - (25) then self.x = game.level.bounds.width - (25) end

	if self.y < (25) then self.y = (25) end
	if self.y > game.level.bounds.height - (25) then self.y = game.level.bounds.height - (25) end

	if self.spin > self.facing then self.facing = math.min(self.facing + dt * self.turnSpeed, 1) end
	if self.spin < self.facing then self.facing = math.max(self.facing - dt * self.turnSpeed, -1) end

	self.anticipation = false
	for i, f in pairs(game.food) do
		local col = math.distanceFrom(self.x, self.y, f.x, f.y)--overlap(self.x - 20, self.y - 20, 40, 40,  f.x - 5, f.y - 15, 10, 30)
		if col < ((self.rad + f.rad)) then
			if math.abs(self.y - f.y) < ((self.rad + f.rad)) then self.anticipation = false end
			if f.x < self.x then self.spin = 1 end
			if f.x > self.x then self.spin = -1 end
			f:collect(1)
			self.frame = #self.animationFrameList
			print("OVERLAP!!!", f.id)
		elseif col < ((self.rad + 60)) and self.frame < 5 then
			self.anticipation = true
			self.anticipateFood = i
		end
	end
end

function playerObject:draw()
	local n = self.animationFrameList[math.floor(self.frame)]
	if self.anticipation then
		n = 5
		--self.rotation = -(math.findAngle(self.x, self.y, game.food[self.anticipateFood].x, game.food[self.anticipateFood].y, true) - math.rad(180))
	end
	local s = 0.3
	lgr.setColor(255,255,255)
	draw(self.spriteSheet, self.quadList[n], (self.x), (self.y + self.bob), self.rotation + self.weave, self.facing * s, s, 100, 100)
	-- lgr.setColor(255,0,255,100)
	-- lgr.circle("fill", self.x, self.y, self.rad)
end
